close all;
clear;
clc;

% Resistor Value Calculator:

% Welcome Message & Menu Options
fprintf('Welcome to the Resistor Value Calculator!\n\n');
fprintf('Please select the type of resistor you have:\n');
fprintf('\t1. 4-Band Resistor\n');
fprintf('\t2. 5-Band Resistor\n');
fprintf('\t0. Exit\n');

% Prompt user for menu selection
while true
    selection = input('\nEnter your selection (1, 2, or 0): ');
    if selection == 1 || selection == 2
        break;
    elseif selection == 0
        return;
    else
        % Throw an error for invalid selection, and restart the program.
        fprintf('\nInvalid selection, please enter 1 or 2.\n');
        input('Press Any Key to restart.');
        eval([mfilename, ';']);
        return;
    end
end

% Prompt user for colour bands
if selection == 1
    fprintf('\nEnter the colours of your 4 bands (in order):\n');
else
    fprintf('\nEnter the colours of your 5 bands (in order):\n');
end


if selection == 1
    bands = {'','','','default'};
else
    bands = {'','','','','default'};
end

% Prompt for and validate each 'digit' colour band
for i = 1:selection+1
    prompt = sprintf('Band %d Colour: ', i);
    while true
        colour = lower(input(prompt, 's'));
        if ~any(strcmp(colour, {'black', 'brown', 'red', 'orange', 'yellow', 'green', 'blue', 'violet', 'gray', 'white', 'gold', 'silver'}))
            fprintf('Invalid Colour, please enter a valid colour.\n');
        else
            break
        end
    end
    bands{i} = colour;
end

% Prompt for and validate 'multiplier' colour band
prompt = sprintf('Multiplier Band Colour: ');
while true
    colour = lower(input(prompt, 's'));
    if ~any(strcmp(colour, {'black', 'brown', 'red', 'orange', 'yellow', 'green', 'blue', 'gold', 'silver'}))
        fprintf('Invalid Colour, please enter a valid colour.\n');
    else
        break
    end
end
bands{selection+2} = colour;

% Prompt for and validate 'tolerance' colour band
prompt = sprintf('Tolerance Band Colour: ');
while true
    colour = lower(input(prompt, 's'));
    if ~any(strcmp(colour, {'brown', 'red', 'gold', 'silver','default'}))
        fprintf('Invalid Colour, please enter a valid colour.\n');
    else
        break
    end
end
bands{selection+3} = colour;

% Calculate resistance value and tolerance
if selection == 1
    resistance = get_resistance({bands{1}, bands{2}});
    resistance = resistance * get_multiplier(bands{3});
    tolerance = get_tolerance(bands{4});
else
    resistance = get_resistance({bands{1}, bands{2}, bands{3}});
    resistance = resistance * get_multiplier(bands{4});
    tolerance = get_tolerance(bands{5});
end

% Display result to user
fprintf('\nResistance value: %gΩ\n', resistance);
fprintf('Tolerance: %s\n', tolerance);

% Prompt user to calculate another resistor or exit
while true
    again = input('\nCalculate another resistor? (y/n): ', 's');
    if strcmpi(again, 'y')
        eval([mfilename, ';']);
        return;
    elseif strcmpi(again, 'n')
        return
    else
        fprintf('Invalid selection, please enter y or n.\n');
    end
end

% Helper Functions used above
function resistance = get_resistance(bands)
    colours = {'black','brown','red','orange','yellow','green','blue','violet','grey','white'};
    digits = [0,1,2,3,4,5,6,7,8,9];
    resistance = 0;

    % Build the Resistance digit by digit
    for i = 1:length(bands)
        resistance = resistance*10 + digits(strcmp(colours,bands{i}));
    end
end


function multiplier = get_multiplier(colour)
    colours = {'black','brown','red','orange','yellow','green','blue','gold','silver'};
    multipliers = [1,10,100,1000,10000,100000,1000000,0.1,0.01];
    
    % Find & Look up multiplier value in the array
    multiplier = multipliers(strcmp(colours,colour));
end

function tolerance = get_tolerance(colour)
    colours = {'brown','red','gold','silver','default'};
    tolerances = [1,2,5,10,20];

    % Find & Look up tolerance value in the array
    tol = tolerances(strcmp(colours,colour));
    tolerance = '±'+string(tol) +'%';
end
